package jp.rinco.android;

import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Canvas;

public class GameObject {
	public int w, h, x, y, z; // Rect in GameScreen
	public int dx, dy, dxm, dym; // Speed & Speed Max
	public Rect r; // Collision Rect
	public int rpw, rph; // Collision Padding
	public int cw, ch, cx, cy; // Rect in Canvas

	public GameObject(int _w, int _h) {
		r = new Rect();
		init(_w, _h);
	}
	void init(int _w, int _h) {
		w = _w;
		h = _h;
		x = y = z = dx = dy = rpw = rph = 0;
		dxm = _w / 2;
		dym = _h / 2;
		calc();
	}
	public GameObject calc() {
		r.left = x + rpw;
		r.top = y + rph;
		r.right = x + w - rpw * 2;
		r.bottom = y + h - rph * 2;
		return this;
	}
	public Boolean collision(GameObject o) {
		return r.contains(o.r.left, o.r.top) || r.contains(o.r.left, o.r.bottom) || r.contains(o.r.right, o.r.top) || r.contains(o.r.right, o.r.bottom);
	}

	public GameObject move() {
		x += dx;
		y += dy;
		return this;
	}
	public GameObject accelerate(int x, int y) {
		dx += x;
		dx = dx < -dxm ? -dxm : dx > dxm ? dxm : dx;
		dy += y;
		dy = dy < -dym ? -dym : dy > dym ? dym : dy;
		return this;
	}
	public GameObject draw(Canvas c) {
		Paint paint = new Paint();
		paint.setColor(Color.RED);
		c.drawRect(new Rect(cx, cy, cx + cw, cy + ch), paint);
		return this;
	}

	// --------- Work with GameObject ----------
/*	public GameObject warp(GameScreen s) {
		s.warp(this);
		return this;
	}
	public GameObject transform(GameScreen s) {
		s.transform(this);
		return this;
	} */
	public GameObject m(GameScreen s) {
		move();
		s.warp(this);
		s.transform(this);
		return this;
	}
}
