package jp.rinco.android.spacetrip;

import java.util.ArrayList;
import java.util.List;


import jp.rinco.android.GameBitmapObject;
import jp.rinco.android.GameObject;
import jp.rinco.android.GameScreen;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.SurfaceView;

public class GameActivity extends Activity {
	GameView view;
	Thread mainThread;

	class GameView extends SurfaceView implements Runnable {
		Context context;
		Paint paint = new Paint();
		Boolean isRunning;
		GameScreen s;
		List<GameObject> rocks = new ArrayList<GameObject>();
		GameBitmapObject ship;
		long score, loopCount;

		public GameView(Context c) {
			super(c);
			context = c;
			mainThread = new Thread(this);
			mainThread.start();
		}
		void init() {
			paint.setColor(Color.WHITE);
			score = loopCount = 0;
			s.setup(ship);
			s.center(ship);
			ship.dx = ship.dy = 0;
			ship.rpw = s.w(0.01F);
			ship.rph = s.h(0.01F);
			rocks.clear();
			addRock();
			addRock();
			addRock();
		}

		void sleep(int v) {
			try {
				Thread.sleep(v);
			} catch (InterruptedException e) {}
		}
		public void run() {
			Canvas canvas;
			do {
				sleep(100);
				canvas = getHolder().lockCanvas();
				Log.d("GameActivity", "LOOP");
			} while (canvas == null);

			// ---------- Init ----------
			s = GameScreen.factory().init(canvas);
			getHolder().unlockCanvasAndPost(canvas);

			Bitmap b = BitmapFactory.decodeResource(context.getResources(), R.drawable.droid);
			ship = new GameBitmapObject(s.w(0.1F), s.h(0.1F), b);
			init();

			// ---------- Main Loop ----------
			while (true) {
				if (isRunning) {
					ship.m(s).calc();
					for (GameObject rock : rocks) {
						rock.m(s).calc();
					}
					doDraw();
					for (GameObject rock : rocks) {
						if (ship.collision(rock)) {
							gameover();
							break;
						}
					}
					score += Math.abs(s.scaleX(ship.dx)) + Math.abs(s.scaleX(ship.dy));
					if (rocks.size() -3  < ++loopCount / 100)
						addRock();
				}
				sleep(100);
			}
		}
		void doDraw() {
			Canvas canvas = getHolder().lockCanvas();
			if (canvas != null) {
				canvas.drawColor(Color.BLACK);
				String sc = "0000000000" + score;
				canvas.drawText(sc.substring(sc.length() - 10), 0, paint.getTextSize(), paint);
				ship.draw(canvas);
				for (GameObject rock : rocks) {
					rock.draw(canvas);
				}
				getHolder().unlockCanvasAndPost(canvas);
			}
		}
		void addRock() {
			Bitmap b;
			if (rocks.size() > 0)
				b = ((GameBitmapObject)rocks.get(0)).bitmap;
			else
				b = BitmapFactory.decodeResource(context.getResources(), R.drawable.redrock0);
			GameBitmapObject o = new GameBitmapObject(s.w(0.01F * s.i(10, 5)), s.h(0.01F * s.i(10, 5)), b);
			o.rpw = s.w(0.01F);
			o.rph = s.h(0.01F);
			s.setup(o);
			do {
				s.random(o);
				o.calc();
			} while (ship.collision(o));
			o.dx = s.w(0.001F * s.i(80, -39));
			o.dy = s.h(0.001F * s.i(80, -39));
			rocks.add(o);
		}
		public void gameover() {
			isRunning = false;
			Intent intent = new Intent(GameActivity.this, GameOverActivity.class);
			intent.putExtra("SCORE", score);
			startActivity(intent);
			init();
		}
		@Override
		public boolean onTouchEvent(MotionEvent event) {
			//Log.d("MyGame", "onTouchEvent()" + s.w(0.1F));
			int x = ship.cx + ship.cw / 2;
			int y = ship.cy + ship.ch / 2;
			ship.accelerate((int)(event.getX() - x) * s.w(0.0001F), (int)(event.getY() - y) * s.h(0.0001F));
			return true;
		}

	}
	@Override
	protected void onCreate(final Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		Log.d("GameActivity", "onCreate()");
		setTitle("Space Trip");
		view = new GameView(this);
		setContentView(view);
	}
	protected void onStart() {
		super.onStart();
		Log.d("GameActivity", "onStart()");
	}
	protected void onResume() {
		super.onResume();
		Log.d("GameActivity", "onResume()");
		view.isRunning = true;
	}
	protected void onPause() {
		super.onPause();
		Log.d("GameActivity", "onPause()");
		view.isRunning = false;
	}
	@Override
	protected void onDestroy() {
		super.onDestroy();
		Log.d("GameActivity", "onDestroy()");
		if (mainThread != null && mainThread.isAlive()) {
			mainThread.interrupt();
		}
	}

}
